# -----------------------------
# DigiTech RP-Series Patch Editor
# Bruce James 
# Copyright (c) 1997 - 2003
# -----------------------------


# -----------------------------
# Description:
#
# The Application
# 
# -----------------------------

namespace eval App {

  variable _App

  # -----------------------------
  # Exported methods

  proc Run { RootDir FileName DevMode } {

    # -----------------------------
    # Application Initialisation
    # -----------------------------
    variable _App

    set _App(Version) "2.2a"
    set _App(Date) "18 January, 2003"

    set _App(DevMode) $DevMode

    # Directory paths
    set _App(RootDir) $RootDir
    set _App(LibDir) [file join $RootDir Lib]
    set _App(ExtendDir) [file join $RootDir Extend]
    set _App(ImagesDir) [file join $RootDir Images]
    set _App(SettingsDir) [file join $RootDir Settings]
    set _App(TemplatesDir) [file join $RootDir Templates]
    set _App(HelpDir) [file join $RootDir Help]
    set _App(DefaultsDir) [file join $RootDir Defaults]
    set _App(PatchesDir) [file join $RootDir Patches]

    set _App(RpmidiLoaded) 0
    set _App(RpdirpkrLoaded) 0

    set _App(FirstTimeRun) 1
    set _App(ShowDevModeWarning) 1

    wm withdraw .
    ::Cursor::Set 1


    # Set general options - fonts, padding
    option add *Font {Helvetica 9 normal}
    option add *Button.padY 0
    option add *Button.borderWidth 1

    # Application fonts
    set _App(SmallFont) {Helvetica 8 normal}
    set _App(NormalFont) {Helvetica 8 normal}
    set _App(EditorFont) {Courier 10 normal}
    set _App(CanvasHeadingFont) {Arial 8 {bold italic}}
    set _App(CanvasLabelFont) {Arial 8 normal}

    # Add Windows bindings
    bind Text <Control-x> "event generate %W <<Cut>>"
    bind Text <Control-c> "event generate %W <<Copy>>"
    bind Text <Control-v> "event generate %W <<Paste>>"
    bind Entry <Control-x> "event generate %W <<Cut>>"
    bind Entry <Control-c> "event generate %W <<Copy>>"
    bind Entry <Control-v> "event generate %W <<Paste>>"

    # Set up transparent gif colour
    label .temp
    global TRANSPARENT_GIF_COLOR
    set TRANSPARENT_GIF_COLOR [.temp cget -bg]
    destroy .temp


    # Splash Screen
    ::Splash::Show 0


    # Set up images
    foreach i {lk nolk dfltsave spindown spinup menudown plain checkon checkoff checkhighlight} {
      image create photo $i -file [file join $_App(ImagesDir) Misc $i.gif] -format gif
    }
    foreach i {commnull commsd commrc commls commtk commlstk} {
      image create photo $i -file [file join $_App(ImagesDir) Misc $i.gif] -format gif
    }
    foreach {i f} {Sine sine Triangle triangle Logarithmic log Exponential exp SP1 sp1 SP2 sp2 SP3 sp3} {
      image create photo LFOWaveform_$i -file [file join $_App(ImagesDir) Modules $f.gif] -format gif
    }
    for {set i 1} {$i <= 10} {incr i} {
      image create photo ERShape_$i -file [file join $_App(ImagesDir) Modules ershp$i.gif] -format gif
    }
    foreach {i f} {Decaying rvdecay Flat rvflat Reverse rvrevrse} {
      image create photo RvbGatedEnvelope_$i -file [file join $_App(ImagesDir) Modules $f.gif] -format gif
    }
    for {set i 0} {$i <= 25} {incr i} {
      image create photo SD2_$i -file [file join $_App(ImagesDir) Modules SD2_$i.gif] -format gif
    }
    foreach i { \
      B1x1 1x1 1x2 2x1 2x2 2x3 3x1 3x2 3x3 4x1 4x2 4x3 5x1 5x2 5x3 6x2 7x2 8x2 \
      M1x2 M1x4 M2x2 2TDly 4TDly Duc MMix MonoFx DualFx StereoFx NGt Pan Morph Morph2 Output Summer Amp} {
      image create photo Button_$i -file [file join $_App(ImagesDir) Buttons $i.gif] -format gif
    }
    foreach i { links lfo1 lfo2 dyn1 dyn2 copy cut fopen fsave gen listen msglog options \
                panel utilitysettings parse paste reqcurr request sap send sendcurr talk } { 
      image create photo Tool_$i -file [file join $_App(ImagesDir) ToolBar $i.gif] -format gif
    }


    # Load the extension DLL's (Windows only)
    global tcl_platform
    if { $tcl_platform(platform) == "windows" } {
    # Load the Tcl MIDI SysEx extension
      if { [info commands midi] != "" } {
        set _App(RpmidiLoaded) 1
      } else {
        if { [catch {
          load [file join $_App(ExtendDir) rpmidi.dll] Rpmidi
          } ] == 0 } {
          set _App(RpmidiLoaded) 1
        }
      }

      # Load the Tcl Directory picker extension
      if { [info commands tk_getdir] != "" } {
        set _App(RpdirpkrLoaded) 1
      } else {
        if { [catch {
          load [file join $_App(ExtendDir) rpdirpkr.dll] Rpdirpkr
          } ] == 0 } {
          set _App(RpdirpkrLoaded) 1
        }
      }
    }

    ::SettingsMgr::Init

    ::Dev::Init
    ::File::Init
    ::Windows::Init
    ::Comm::Init
    ::Param::Init
    ::DialogMgr::Init
    ::Options::Init
    ::UtilitySettings::Init
    ::SyxEdit::Init
    ::SAPEdit::Init
    ::Panel::Init
    ::Browse::Init
    ::Description::Init
    ::DefaultSave::Init
    ::ProgramMenu::Init
    ::ProgramPicker::Init
    ::ProgramNames::Init
    ::CaptureSysEx::Init
    ::Help::Init
    ::Welcome::Init
    ::Device::Init
    ::Main::Init

    ::DetermineDefaults::Init
    ::DumpPrograms::Init
    ::DumpUserDefaults::Init

    ::SettingsMgr::Get

    ::Main::Create
    ::SAPEdit::Create
    ::Description::Create

    ::SettingsMgr::Subscribe "::App::PutSettingsCommands"

    if { [::App::Member DevMode] != {} && $_App(ShowDevModeWarning) } {
      if { [tk_messageBox -title "Development Mode" -icon question -type yesno -message \
        "Note that RPEdit is about to start in development mode.  This mode provides additional \
commands and functions used in the development and testing of RPEdit.  These may be \
confusing, incomplete, and impact RPEdit's performance.  Use this mode only if you need to.\n\n\
Development mode is started by the presence of a non-empty, <optional-development-mode> parameter on \
the command line:\n
Usage: Wishrun <RPEdit-path>RPEdit.tcl <optional-filename> <optional-development-mode>\n\n\
If you unintentionally started RPEdit in this mode, check the command line for long filenames \
that are not properly quoted.\n\n\
Do you want RPEdit to continue in Development Mode?" ] == "yes" } {
        set _App(ShowDevModeWarning) 0
      } else {
        set _App(DevMode) {}
      }
    }


    # -----------------------------
    # Main Dialog
    # -----------------------------

    # Show main dialog
    ::Main::Show


    # Open file if specified on command line
    if { $FileName != {} } {
      ::File::Open $FileName
    }
    ::SyxEdit::SetOriginalFileState
    ::SAPEdit::SetOriginalFileState

    ::Main::Deiconify

    catch {destroy .splash}

    update
    ::Cursor::Set 0

    ::Main::TakeFocus

    if { $_App(FirstTimeRun) } {
      ::Welcome::Show
    }

    # Action Startup options
    set CommDeviceNo [::Options::Param Device]
    set CommDeviceClass ::[::Device::DeviceToClassId $CommDeviceNo]::
    if { [::App::Member RpmidiLoaded] && [expr [::Options::Param MIDIInDevice] != 0 && [::Options::Param MIDIOutDevice] != 0] } {
      if { [::Options::Param StartupListenMode] } {
        ::Comm::Listen 0
      }
      if { [::Options::Param StartupTalkMode] && [${CommDeviceClass}AllowsTalk $CommDeviceNo] } {
        ::Comm::Talk 0
      }
      if { [::Options::Param StartupRequestCurrent] && [${CommDeviceClass}AllowsProgramRequest $CommDeviceNo] } {
        # Request current program, provided no file on command line
        if { $FileName == {} } {
          ::ProgramMenu::RequestCurrent
        }
      }
      if { [::Options::Param StartupRequestProgramNames] && [${CommDeviceClass}AllowsProgramDirectory $CommDeviceNo] } {
        ::ProgramNames::PreLoad
      }
      if { [::Options::Param StartupRequestUtilitySettings] && [${CommDeviceClass}AllowsUtilitySettings $CommDeviceNo] } {
        ::UtilitySettings::RequestCurrent
      }
    }
  }


  proc PutSettingsCommands {} {
    variable _App

    lappend Cmds "set ::App::_App(FirstTimeRun) 0"
    lappend Cmds "set ::App::_App(ShowDevModeWarning) $::App::_App(ShowDevModeWarning)"

    return $Cmds
  }


  proc Member { MemberName } {
    variable _App
    return $_App($MemberName)
  }


  proc Assert { Condition Message } {
    if { [expr !($Condition)] } {
      error "Assert: $Condition: $Message"
    }
  }


}

