# -----------------------------
# DigiTech RP-Series Patch Editor
# Bruce James 
# Copyright (c) 1997 - 2003
# -----------------------------


# -----------------------------
# Description:
#
# Help displays the Help dialog.
# 
# -----------------------------


namespace eval Help {

  # -----------------------------
  # Exported methods

  proc Init {} {
  }


  proc ShowHelp { {HelpUrl "Help.html" } } {

    ShowUrl [file join [::App::Member HelpDir] $HelpUrl]
  }


  proc ShowUrl { Url } {

    # Split out executable from the rest of the arguments
    set HelpViewer [::Options::Param HelpViewer]
    set HelperApp {}
    set Args {}
    if { $HelpViewer != {} } {
      global tcl_platform
      if { $tcl_platform(platform) == "windows" } {
        # Allow for long, quoted filenames
        if { [regexp -indices {^ *\"(.+)\"( |$)} $HelpViewer Dummy HelperAppIndices] } {
          set HelperApp [string range $HelpViewer [lindex $HelperAppIndices 0] [lindex $HelperAppIndices 1]]
          set Args [string range $HelpViewer [expr [lindex $HelperAppIndices 1] + 2] end]
        }
      }
      if { $HelperApp == {} } {
        if { [regexp -indices {^ *([^ ]+)( |$)} $HelpViewer Dummy HelperAppIndices] } {
          set HelperApp [string range $HelpViewer [lindex $HelperAppIndices 0] [lindex $HelperAppIndices 1]]
          set Args [string range $HelpViewer [expr [lindex $HelperAppIndices 1] + 1] end]
        }
      }
    }
    if { [file exists $HelperApp] == 0 } {
      tk_messageBox -icon error -type ok -title "Help Viewer" \
              -message "The web browser either is not defined or does not exist. \
Please use \"View/Options/General\" to set a web browser for viewing help."
      return
    }

    cd [::App::Member SettingsDir]
    eval exec "\{$HelperApp\}" $Args $Url >Temp.log &  
  }


  proc Report {} {

    set GeneratedTime [clock seconds]
    if { [catch {
        set File [file join [::App::Member SettingsDir] Report_[clock format $GeneratedTime -format "%d%b%y_%H%M%S"].txt]
        set ReportFile [open $File w]
      } ] != 0 } {
      tk_messageBox -icon error -type ok \
              -message "Could not create report."
      return
    }

    global tcl_platform tcl_version tcl_patchLevel
    puts $ReportFile "
=======================================================================
RPEdit Problem Report
=======================================================================
File:                 $File
Generated:            [clock format $GeneratedTime]

RPEdit Version:       [::App::Member Version]
Tcl/Tk Version:       $tcl_patchLevel
Operating System:     $tcl_platform(os), $tcl_platform(osVersion)
Root directory:       [::App::Member RootDir]
DevMode:              [::App::Member DevMode]
Extensions loaded:    [::App::Member RpmidiLoaded], [::App::Member RpdirpkrLoaded]
Connected Device:     [::Device::Params::Device::MIDIToValue "_" [::Options::Param Device]]
Support new modules:  [::Options::Param MIDINewModules]
MIDI In/Out Devices:  [::Options::Param MIDIInDevice], [::Options::Param MIDIOutDevice]

-------------------
What device were you using and what is its version number (e.g. RP-21D v1.1):


Describe the problem you encountered giving as much detail as you can
(e.g. what happened, what actions led to this, what error messages, etc.):




-------------------
This report is located at, $File.
Send this report, along with any relevant files, to:
   rpedit@mail.netmails.net

Note that the RPEdit log file, located at, [file join [::App::Member SettingsDir] RPEdit.log], 
may also contain information useful in diagnosing this problem.  You may 
wish to send this file as well.  If the file is large (>100K), then use a text editor to 
delete out all but the last 200 - 300 lines.
"

    catch {close $ReportFile}

    # Split out executable from the rest of the arguments
    set TextEditor [::Options::Param TextEditor]
    set HelperApp {}
    set Args {}
    if { $TextEditor != {} } {
      global tcl_platform
      if { $tcl_platform(platform) == "windows" } {
        # Allow for long, quoted filenames
        if { [regexp -indices {^ *\"(.+)\"( |$)} $TextEditor Dummy HelperAppIndices] } {
          set HelperApp [string range $TextEditor [lindex $HelperAppIndices 0] [lindex $HelperAppIndices 1]]
          set Args [string range $TextEditor [expr [lindex $HelperAppIndices 1] + 2] end]
        }
      }
      if { $HelperApp == {} } {
        if { [regexp -indices {^ *([^ ]+)( |$)} $TextEditor Dummy HelperAppIndices] } {
          set HelperApp [string range $TextEditor [lindex $HelperAppIndices 0] [lindex $HelperAppIndices 1]]
          set Args [string range $TextEditor [expr [lindex $HelperAppIndices 1] + 1] end]
        }
      }
    }
    if { [file exists $HelperApp] == 0 } {
      tk_messageBox -icon error -type ok -title "Help Viewer" \
              -message "The text editor either is not defined or does not exist. \
Please use \"View/Options/General\" to set a text editor for editing text files.

Report has been saved to the file, $File.
"
      return
    }

    if { [regsub ".*(\"?%1\"?).*" $Args "\"$File\"" NewArgs] > 0 } {
      set Args $NewArgs
    } else {
      set Args "\"$File\" $Args"
    }
    cd [::App::Member SettingsDir]
    eval exec "\{$HelperApp\}" $Args >Temp.log &
  }

}

